"""
This package contains patterns which are modifiers for existing patterns. If
a pattern takes another pattern in its constructor, it belongs in this package.
"""

from .strobe import Strobe

__all__ = ['Strobe']
